@@ -17,6 +17,9 @@ geo_geocodio <- function(...) geo(method = 'geocodio', ...)
 #' @export
 geo_iq <- function(...) geo(method = 'iq', ...)
 
+#'@rdname geo_census
+#' @export
+geo_google <- function(...) geo(method = 'google', ...)
 
 #' Convenience function for calling the \code{\link{geo}} function
 #' with method = 'cascade'
@@ -65,7 +68,7 @@ geo_cascade <- function(..., cascade_order = c('census', 'osm')) {
           input[!names(input) %in% c(pkg.globals$address_arg_names, 'method', 'unique_only')],
             list(method = cascade_order[2], unique_only = FALSE)))
     
-    ## combine census and retry results
+    ## combine initial and retry results
     combi <- initial_results
     combi[na_indices,] <- retry_results
     combi$geo_method <- ifelse(na_indices, cascade_order[2], cascade_order[1])
